%% Demo_NIPS2011: The startup script

% This demo provides an example implementation to construct the Portmanteau
% vocabularies. Here we combine color and shape features. The running time
% of the algorithm depend on the size of color and shape vocabularies. With
% larger vocabularies expect this algorithm to take several hours to finish. 
% 

%Literature:

% Fahad Shahbaz Khan, Joost van de Weijer, Andrew D. Bagdanov and Maria Vanrell
% Portmanteau Vocabularies for Multi-Cue Image Representations, NIPS2011
% For any questions, please email to fahad@cvc.uab.cat.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Load the data and initializing parameters training data
Init_Birds
labels=cell2mat(struct2cell(birdsopts.labels));
trainset=cell2mat(struct2cell(birdsopts.trainset));

histogram_Shape=cell2mat(struct2cell(birdsopts.histogram_Shape));
histogram_Color=cell2mat(struct2cell(birdsopts.histogram_Color));

Shape_Index=birdsopts.Shape_Index;
Color_Index=birdsopts.Color_Index;

%%  Specify the compression size
compress_size=500;
% Alpha=0.7;  %%%% The weighting parameter (to levearge the contribution of visual cues)
% Beta=5;     %%%% to obtain highly discriminative clusters ( we put the beta to 5 for all our experiments)

%% compute the joint-distributions for color and shape
Pcw_Shape=Do_compute_Pcw(histogram_Shape,trainset,labels);
% Pcw_Shape=Pcw_Shape.^Alpha.Wf1;

Pcw_Color=Do_compute_Pcw(histogram_Color,trainset,labels);
% Pcw_Color=Pcw_Color.^(1-Alpha.Wf1);

%% Compute the product of the two color and shape distributions
Pcw_Product=Do_Product_Distributions(Pcw_Shape,Pcw_Color);
% Pcw_Product=Pcw_Product.^Beta;
% Pcw_Product=Pcw_Product./sum(Pcw_Product(:));

%% Intialization of DITC Clusters
[W]=Do_Initialization(Pcw_Product,compress_size); 

%% Perform DITC based Compression to obtain Portmanteau word-indices
[Portmanteau_Words]=Do_DITC_Portmanteau(Pcw_Product,W,compress_size);

%% Reassignment based on Portmanteau words
Portmanteau_Histogram=Do_Compute_Histogram(Portmanteau_Words,Shape_Index,Color_Index,compress_size,size(histogram_Shape,1),size(histogram_Color,1));

%% Classification
% For classification install libsvm (www.csie.ntu.edu.tw/~cjlin/libsvm/) and a non linear kernel such as 
% Intersection kernel (http://www.cs.berkeley.edu/~smaji/). In our
% experiments we use a chi-square kernel.

%  [predict_label, dec_values]=Do_Classification_Portmanteau      %(Portmanteau_Histogram,birdsopts);  %%% uncomment the line to do SVM 