%Do_DITC_Portmanteau: optimizes the initally created word clusters (W),
%during the initialization process, and returns both of
%the optimized word clusters (W) along with the final compressed histogram.

%SYNOPSIS:
%     [hist_final,W]=Do_DITC(Pcw,W,PcW)

%INPUT:
%  Pcw: the probability of each word (w) towards each category(c)
%  W:   the intially created word clusters (W)
%  compress_size: the required compression size

%OUTPUT:
% InitClusters:            the optimized compact word clusters

%Literature:
% Code written by Noha Elfiky:
% Noha M. Elfiky, Fahad Shahbaz Khan, Joost van de Weijer,Jordi Gonza`lez
% Discriminative Compact Pyramids for Object and Scene Recognition
% Pattrn Recognition, accepted 2011

function [InitClusters]=Do_DITC_Portmanteau(Pcw,W,compress_size)

stop=0;  prevObjFun = 100;
Pw=sum(Pcw,2); %Pw: set of all word priors
nclasses=size(Pcw,2);
no_words=size(Pcw,1);
KL=zeros(no_words,compress_size);
PcW=zeros(compress_size,nclasses);    %Relates the new compressed clusters to the dataset categories
Pcw = normalize(Pcw, 2);

while(stop==0)
    %%step2: for each cluster Wj (desired ones), compute
    PWj=zeros(compress_size,1);
    for wj=1:compress_size
        PWj(wj,1)=sum(Pw(W{wj},1));
        div_priors=(Pw(W{wj},1)./(PWj(wj,1)+eps));
        tmp=repmat(div_priors,1,nclasses).*Pcw(W{wj},:);
        PcW(wj,:)=sum(tmp,1);
    end
    %%step3:
    %For each word wt, find its new cluster index
    PcW = normalize(PcW, 2);
    %For each Word cluster:
    for kk=1:compress_size
        %Get the distribution of the current word cluster to the classes
        div1=(ones(size(Pcw,1),1)*PcW(kk,:)+eps^2);
        %Calculate the Relative between the original features and the current word cluster
        div2=Pcw.*log2((Pcw./div1)+eps^2);
        %Relate the original features to the newly created word CLusters
        KL(:,kk)=sum(div2,2);
    end
    %Compute the new word cluster for the original features(words)
    [Valuesnew,InitClusters]=min(KL,[],2);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %stopping criteria    
    ObjFn=sum(Pw.*Valuesnew);
    
    %visualize the optimization convergence
    (prevObjFun - ObjFn)
    
     %Terminition condition
    if((prevObjFun - ObjFn) < 0.001) 
        stop = 1;
    else
        stop = 0;
        prevObjFun = ObjFn;
    end
    
    %get words per cluster
    for cc=1:compress_size
        [ii,vv]=find(InitClusters==cc);
        %save the words inside the newly created compact Word clusters W
        W{cc}=ii;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Remove Empty bins:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

emptyCluster=zeros(1,compress_size);
%%assign words to clusters
for cc=1:compress_size
    [ii,vv]=find(InitClusters==cc);
    if(sum(vv)==0)
        sprintf('Empty cluster: %d',cc);
        %save a list of empty clusters
        emptyCluster(1,cc)=1;
        %Resolve the issue of the empty bins
        %Valuesnew>contains the min values retutrn by teking the min over KL matrix
        [ii,InitClusters,Valuesnew]=FillEmptyClusters(InitClusters,Valuesnew,cc);
    end;
    %save the words inside the newly created compact Word clusters W
    W{cc}=ii;
end

end