%Do_Initialization: performs initial clustering
%of the input histogram words (w) to the required
% Word clusters (W) of size (compress_size).

%SYNOPSIS:
%   [W]=Do_Initialization(histogram,Pcw,compress_size)

%INPUT:

%  Pcw: the probability of each word (w) towards each category(c)
%  compress_size: the required compression size

%OUTPUT:
%  W:     the inital word clusters, which are not optimized yet

% Code written by Noha Elfiky:
% Noha M. Elfiky, Fahad Shahbaz Khan, Joost van de Weijer,Jordi Gonza`lez
% Discriminative Compact Pyramids for Object and Scene Recognition
% Pattrn Recognition, accepted 2011

function [W]=Do_Initialization(Pcw,compress_size)
%% Input
%number of classes in the dataset
nclasses=size(Pcw,2);%%l:number of classes
no_words=size(Pcw,1);%number of features
counter=1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%START INITIALIZATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%dimension should be=words*1(new cluster index)
[val,InitClusters]=max(Pcw,[],2);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%START Splitting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%We split the initial l word clusters to small clusters
%relative to the capacity of the current word cluster at hand
clusters_sizes = [];
for c=1:nclasses%%for each cluster of the l initial ones
    %find the words in this cluster
    %vv: boolean that shows if the words belongs or not to the cluster c
    %ii: the index value
    %InitClusters: are the initial words after their assignment to l initial clusters
    [ii,vv]=find(InitClusters==c);%% output and dimension is to be checked!!!!!!
    %no of words to be split
    no_words_per_cluster=length(ii);
    if(no_words_per_cluster == 0)
        continue;
    end
    %The expansion of the initial Word clusters to the requied size k
    %will depend on the capacity of words in that cluster.
    no_subClusters= ceil(compress_size*(no_words_per_cluster/no_words) );
    %store the cluster divisions
    clusters_sizes = [clusters_sizes no_subClusters];
    cluster_sub{c}=no_subClusters;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Here we see the distribution of words (x) with in each subcluster
    x = floor(no_words_per_cluster/ no_subClusters);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Show beside each subcluster word; the subcluster index it belongs to
    IDX = zeros(1, no_words_per_cluster);
    for iidx = 1:no_subClusters - 1
        IDX(((iidx - 1) * x + 1):iidx*x) = iidx;
    end
    %Each subcluster in the cluster at hand will contain x words
    %Thus, we partition every x words to belong to a subcluster
    %for example, first x words will belong to the first subcluster
    IDX(((no_subClusters - 1) * x + 1):end) = no_subClusters;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %know the actual index for the words in the subcluster at hand
    for t=1:no_subClusters %say we have t subclusters per each  cluster of the l initials
        %a: is a boolean flag
        %b: contains the index value
        [a,b]=find(IDX==t);
        %%get the actual index of the words in the current sub cluster
        words_cluster{t}=ii(b);%% ii contains the words belongings to cluster l
        W{counter}=words_cluster{t};
        counter=counter+1;
    end
    clear words_cluster;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Merging 1:
while(size(W, 2) > compress_size)
    [mv mi] = max(clusters_sizes);
    wInd = sum(clusters_sizes(1:mi-1));
    %Reduce the number of generated clusters  by 1
    clusters_sizes(mi) = mv - 1;
    %Merge the word indices of the generated clusters
    W{wInd + 2} = [W{wInd + 1};W{wInd + 2}];
    %Remove the merged one
    W = {W{1:wInd} W{wInd + 2:end}};
end
end